@@ -5,7 +5,7 @@ del get_versions
 
 from gisutils.projection import get_proj_str, get_authority_crs
 from gisutils.projection import project as project_fn
-from gisutils.raster import get_values_at_points, get_raster_crs
+from gisutils.raster import get_values_at_points, get_raster_crs, write_raster
 from gisutils.shapefile import df2shp, shp2df, get_shapefile_crs
 
 
