@@ -1,20 +1,21 @@
 """
 Functions for working with rasters.
 """
+from pathlib import Path
 import os
-import collections
 import warnings
 import time
 import fiona
-from fiona.crs import to_string
-from shapely.geometry import box, shape, mapping, Polygon
+from shapely.geometry import box, mapping, Polygon
 from shapely import wkt
-import rasterio
-from rasterio import Affine
-from rasterio.mask import mask
+try:
+    import rasterio
+    from rasterio import Affine
+    from rasterio.mask import mask
+except:
+    rasterio = False
 
 import numpy as np
-import pandas as pd
 from scipy import interpolate
 try:
     from osgeo import gdal
@@ -22,7 +23,7 @@ except:
     gdal = False
 
 from gisutils.projection import project, get_authority_crs, project_raster
-from gisutils.shapefile import shp2df, get_shapefile_crs
+from gisutils.shapefile import get_shapefile_crs, shp2df
 
 
 def get_transform(xul, yul, dx, dy=None, rotation=0.):
@@ -47,6 +48,8 @@ def get_transform(xul, yul, dx, dy=None, rotation=0.):
     -------
     affine.Affine instance
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
     if dy is None:
         dy = -dx
     return Affine(dx, 0., xul,
@@ -67,6 +70,8 @@ def get_raster_crs(raster):
     crs : pyproj.CRS instance
 
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
     with rasterio.open(raster) as src:
         if src.crs is not None:
             crs = get_authority_crs(src.crs)
@@ -76,7 +81,7 @@ def get_raster_crs(raster):
 def get_values_at_points(rasterfile, x=None, y=None, band=1,
                          points=None, points_crs=None,
                          out_of_bounds_errors='coerce',
-                         method='nearest'):
+                         method='nearest', size_thresh=1e9):
     """Get raster values single point or list of points. Points in
     a different coordinate reference system (CRS) specified with a points_crs will be
     reprojected to the raster CRS prior to sampling.
@@ -117,6 +122,15 @@ def get_values_at_points(rasterfile, x=None, y=None, band=1,
         return the raster values at the nearest cell centers. If 'linear',
         scipy.interpolate.interpn is used for bilinear interpolation of values
         between raster cell centers.
+    size_thresh : float
+        Prior to reading any data, the raster size (height * width) is evaluated. If
+        the size is larger than size_thresh, point values are read using
+        :meth:`rasterio.io.DatasetReader.sample` (regardless of the specified method),
+        which gets nearest pixel values without reading the whole dataset into memory.
+        A 32-bit raster of size=1e9 would require approximately 4 GB of memory
+        (at 4 bytes per pixel).
+        By default, 1e9.
+
 
     Returns
     -------
@@ -126,6 +140,8 @@ def get_values_at_points(rasterfile, x=None, y=None, band=1,
     -----
     requires rasterio
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
 
     # read in sample points
     array_shape = None
@@ -157,18 +173,28 @@ def get_values_at_points(rasterfile, x=None, y=None, band=1,
     t0 = time.time()
 
     print("reading data from {}...".format(rasterfile))
+    data = None
     with rasterio.open(rasterfile) as src:
         meta = src.meta
         nodata = meta['nodata']
-        data = src.read(band)
+        size = src.shape[0] * src.shape[1]
+        if size < size_thresh:
+            data = src.read(band)
 
-    if points_crs is not None:
-        points_crs = get_authority_crs(points_crs)
-        raster_crs = get_authority_crs(src.crs)
-        if points_crs != raster_crs:
-            x, y = project((x, y), points_crs, raster_crs)
+        # reproject coordinates if needed
+        if points_crs is not None:
+            points_crs = get_authority_crs(points_crs)
+            raster_crs = get_authority_crs(src.crs)
+            if points_crs != raster_crs:
+                x, y = project((x, y), points_crs, raster_crs)
 
-    if method == 'nearest':
+        if data is None:
+            results = src.sample(list(zip(x, y)))
+            results = np.squeeze(list(results))
+
+    if data is None:
+        pass
+    elif method == 'nearest':
         i, j = src.index(x, y)
         i = np.array(i, dtype=int)
         j = np.array(j, dtype=int)
@@ -215,6 +241,85 @@ def get_values_at_points(rasterfile, x=None, y=None, band=1,
     return results
 
 
+def points_to_raster(points_shapefiles, nodata_value=-99,
+                     data_col='values',
+                     output_resolution=250,
+                     outfile='surface.tif', dest_crs=None):
+    """Interpolate point data to a regular grid using scipy.interpolate.griddata;
+    write results to a GeoTiff.
+
+    Parameters
+    ----------
+    points_shapefiles : shapefile or list of shapefiles
+        Point shapefiles with estimated data, assumed to be on a regular grid.
+    nodata_value : numeric
+        Value in `points_shapefiles` indicating no data
+    data_col : str
+        Field in `points_shapefiles` with estimated data.
+    output_resolution : numeric
+        Cell spacing of the output raster
+    outfile : stf
+        Output GeoTiff
+    dest_crs : obj
+        A Python int, dict, str, or pyproj.crs.CRS instance
+        passed to the pyproj.crs.from_user_input
+        See http://pyproj4.github.io/pyproj/stable/api/crs/crs.html#pyproj.crs.CRS.from_user_input.
+        Can be any of:
+          - PROJ string
+          - Dictionary of PROJ parameters
+          - PROJ keyword arguments for parameters
+          - JSON string with PROJ parameters
+          - CRS WKT string
+          - An authority string [i.e. 'epsg:4326']
+          - An EPSG integer code [i.e. 4326]
+          - A tuple of ("auth_name": "auth_code") [i.e ('epsg', '4326')]
+          - An object with a `to_wkt` method.
+          - A :class:`pyproj.crs.CRS` class
+
+    Notes
+    -----
+
+    """
+
+    df = shp2df(points_shapefiles, dest_crs=dest_crs)
+
+    if dest_crs is None:
+        dest_crs = get_shapefile_crs(points_shapefiles)
+
+    # reshape the values column to a nrow x ncol array; convert invalid values to nans
+    data = df[data_col].values
+    data[data == nodata_value] = np.nan
+
+    # coordinates for the orignal grid (aligned with NHG cell corners)
+    x_points = np.array([g.x for g in df.geometry])
+    y_points = np.array([g.y for g in df.geometry])
+
+    # specifications for a new output_resolution grid aligned with NHG
+    # xul, yul is the cell center of the first cell (in the upper left corner)
+    xul = x_points.min()
+    yul = y_points.max()
+    dxy = output_resolution
+
+    # 1D arrays of x and y coordinates for each column, row
+    x = np.arange(np.min(x_points), np.max(x_points) + dxy, dxy)
+    y = np.arange(np.min(y_points), np.max(y_points) + dxy, dxy)
+
+    # 2D arrays of x and y coordinates for each point
+    X, Y = np.meshgrid(x, y)
+
+    # interpolate the values onto the new grid
+    # using bilinear interpolation
+    # `bounds_error=False` means extrapolated points will be filled with nans
+    results = interpolate.griddata((x_points, y_points), data, (X, Y),
+                                   method='linear')
+    results = np.flipud(results)
+
+    results = np.ma.masked_array(results, mask=np.isnan(results))
+    write_raster(outfile, results, xul=xul, yul=yul,
+                 dx=dxy, dy=dxy, rotation=0., crs=dest_crs,
+                 nodata=-9999)
+
+
 def write_raster(filename, array, xll=0., yll=0., xul=None, yul=None,
                  dx=1., dy=None, rotation=0., proj_str=None, crs=None,
                  nodata=-9999, verbose=False,
@@ -225,7 +330,7 @@ def write_raster(filename, array, xll=0., yll=0., xul=None, yul=None,
 
     Parameters
     ----------
-    filename : str
+    filename : str or pathlib.Path
         Path of output file. Export format is determined by
         file extention.
         '.asc'  Arc Ascii grid
@@ -286,6 +391,8 @@ def write_raster(filename, array, xll=0., yll=0., xul=None, yul=None,
     will have the same number of rows and pixels as the original.
 
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
     t0 = time.time()
     a = array
     # third dimension is the number of bands
@@ -310,7 +417,7 @@ def write_raster(filename, array, xll=0., yll=0., xul=None, yul=None,
             dy = dx
         xul = _xll_to_xul(xll, height * dy, rotation)
         yul = _yll_to_yul(yll, height * dy, rotation)
-    if filename.lower().endswith(".tif"):
+    if str(filename).lower().endswith(".tif"):
         trans = get_transform(xul=xul, yul=yul,
                               dx=dx, dy=-np.abs(dy), rotation=rotation)
 
@@ -337,7 +444,7 @@ def write_raster(filename, array, xll=0., yll=0., xul=None, yul=None,
                 dst.write_mask(~a.mask.transpose(1, 2, 0))
         print('wrote {}'.format(filename))
 
-    elif filename.lower().endswith(".asc"):
+    elif str(filename).lower().endswith(".asc"):
         path, fname = os.path.split(filename)
         fname, ext = os.path.splitext(fname)
         for band in range(count):
@@ -358,7 +465,6 @@ def zonal_stats(feature, raster, out_shape=None,
         from rasterstats import zonal_stats
     except:
         raise ImportError("This function requires rasterstats.")
-
     if not isinstance(feature, str):
         feature_name = 'feature'
     else:
@@ -519,6 +625,8 @@ def clip_raster(inraster, clip_features, outraster,
     kwargs : keyword arguments
         Keyword arguments to rasterio.open for writing the output raster.
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
     if clip_kwargs is None:
         clip_kwargs = {}
     if project_kwargs is None:
@@ -531,8 +639,9 @@ def clip_raster(inraster, clip_features, outraster,
     if clip_features_crs is None:
         clip_features_crs = raster_crs
     # get the clip feature crs from shapefile
-    if isinstance(clip_features, str) and os.path.exists(clip_features):
-        clip_features_crs = get_shapefile_crs(clip_features)
+    if isinstance(clip_features, str) or isinstance(clip_features, Path):
+        if Path(clip_features).exists():
+            clip_features_crs = get_shapefile_crs(clip_features)
     # otherwise if clip feature crs was specified
     else:
         clip_features_crs = get_authority_crs(clip_features_crs)
@@ -587,6 +696,8 @@ def _clip_raster(inraster, features, outraster, clip_kwargs, **kwargs):
     kwargs : keyword arguments
         Keyword arguments to rasterio.open for writing the output raster.
     """
+    if not rasterio:
+        raise ModuleNotFoundError('This function requires rasterio. Please conda install rasterio.')
     # convert the clip_features to geojson
     geoms = get_feature_geojson(features)
     with rasterio.open(inraster) as src:
@@ -634,7 +745,7 @@ def get_feature_geojson(features):
     # clip_features is a single geojson geometry
     elif isinstance(features, dict):
         geoms = [features]
-    elif isinstance(features, str):
+    elif isinstance(features, str) or isinstance(features, Path):
         # clip_features is a single wkt string
         try:
             geoms = [mapping(wkt.loads(features))]
